!!ARBvp1.0

# Constant Parameters
PARAM mvp[4]	= { state.matrix.mvp }; 	# modelview projection matrix
PARAM mv[4]     = { state.matrix.modelview };	 # modelview matrix
PARAM Light = program.env[0];		# Light.xyz contains position, Light.w contains size
PARAM ModelPosition = program.env[1];
PARAM AttenuationFactors = program.env[2];	# contains attenuation factors for constant, linear and quadratic attenuation
PARAM LightObjectSpace = program.env[3];	# Light.xyz in object space

# Per-vertex inputs
ATTRIB inColor = vertex.color; 
ATTRIB inPosition = vertex.position;
ATTRIB inNormal   = vertex.normal;

# Per-vertex outputs
OUTPUT outColor    = result.color;
OUTPUT outPosition = result.position;
OUTPUT Attenuation = result.texcoord[0];
OUTPUT Distance = result.texcoord[1];

TEMP Coordinate, LightVector, R0, R1, R2;

DP4 outPosition.x, mvp[0], inPosition;   # Transform the x component of the per-vertex position into clip-space
DP4 outPosition.y, mvp[1], inPosition;   # Transform the y component of the per-vertex position into clip-space
DP4 outPosition.z, mvp[2], inPosition;   # Transform the z component of the per-vertex position into clip-space
DP4 outPosition.w, mvp[3], inPosition;   # Transform the w component of the per-vertex position into clip-space

# calculate temporary coordinate
SUB Coordinate, Light, inPosition;
SUB Coordinate, Coordinate, ModelPosition;
MUL Coordinate, Coordinate, Light.w;

# calculate attenuation and distance coordinates
ADD Attenuation, Coordinate, 0.5;
MOV Attenuation.w, 1;
MUL Distance, Coordinate, -1;
MOV Distance.w, 1;



# calculate lighting for point light (dot-product for N dot L and constant, linear and quadratic attenuation)
SUB LightVector, LightObjectSpace, inPosition;

# normalize light vector
DP3 R0.w, LightVector, LightVector;					
RSQ R1.w, R0.w;
MUL LightVector.xyz, LightVector, R1.w;

# calculate distance vector - R2 then contains (1, d, d, 1/d)
DST R2, R0.w, R1.w;

# simply dot our distance vector with our attenuation factors to get 1/attenuation
DP3 R1.w, R2, AttenuationFactors;

# take reciprocal to get attenuation - R0.w then contains attenuation
RCP R0.w, R1.w;

# dot Normal with Lightvector to get diffuse lighting
DP3 R1.w, LightVector, inNormal;

# scale inColor with attenuation and dot-product, and store the "rest" in outColor
MUL R2, inColor, R1.w;
MUL outColor, R2, R0.w;

END
